/*
 * Copyright (c) 2002, The EROS Group, LLC and Johns Hopkins
 * University. All rights reserved.
 * 
 * This software was developed to support the EROS secure operating
 * system project (http://www.eros-os.org). The latest version of
 * the OpenCM software can be found at http://www.opencm.org.
 * 
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 * 
 * 3. Neither the name of the The EROS Group, LLC nor the name of
 *    Johns Hopkins University, nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <opencm.h>

void
reply_trace(const Reply *r)
{
  log_trace(DBG_PROTO, "SerTrueName:     %s\n", ser_getTrueName(r));
  log_trace(DBG_PROTO, "Request ID :     %d\n", r->reqID);

  report_to_tracelog(DBG_PROTO);

  TRY {
    if (r->value == NULL) {
      log_trace(DBG_PROTO, "Value:\n");
      sdr_show(r->value);
    }
    else if (opt_TraceProtocol > 2) {
      log_trace(DBG_PROTO, "Value:\n");
      sdr_show(r->value);
    }
    else {
      log_trace(DBG_PROTO, "Value:          %s <%s>\n", 
		ser_getTrueName(r->value),
		r->serType->tyName);
    }
  } DEFAULT(ex) {
    report_to_tracelog(TRACE_NONE);
    RETHROW(ex);
  } END_CATCH;

  report_to_tracelog(TRACE_NONE);
}

OC_bool
reply_check(const void *v)
{
  return TRUE;
}

void
reply_show(const void *ob)
{
  const Reply *r = ob;
  
  report(0, "SerTrueName:     %s\n", ser_getTrueName(r));
  report(0, "Request ID :     %d\n", r->reqID);
  report(0, "Value      :     \n");
  sdr_show(r->value);
}

void
reply_serialize(SDR_stream *strm, const void *ob)
{
  const Reply *r = (Reply *)ob;
  
  sdr_w_u32("reqid", strm, r->reqID);
  sdr_write("value", strm, r->value);
  
} /* end reply_serialize */

void *
reply_deserialize(const DeserializeInfo *di, SDR_stream *strm)
{
  Reply *r = (Reply *) GC_MALLOC(sizeof(Reply));

  ser_init(r, &Reply_SerType, di->ver);
  r->reqID          = sdr_r_u32("reqid", strm);
  r->value          = sdr_read("value", strm); 

  SER_MODIFIED(r);
  return r;
}

void
reply_mark(Repository *r, const void *container, 
	   const void *ob, rbtree *memObs)
{
  assert(container == ob);
  /* Leaf object. Nothing to do */
}

Reply *
reply_create(uint32_t reqid, Serializable *s)
{
  Reply *r = (Reply *) GC_MALLOC(sizeof(Reply));

  ser_init(r, &Reply_SerType, Reply_SerType.ver);

  r->reqID = reqid;
  r->value = s;
  r->channel = NULL;  /* channel will be assigned/created later */

  SER_MODIFIED(r);
  return r;
} /* end reply_create */
