/*
 * Copyright (c) 2002, The EROS Group, LLC and Johns Hopkins
 * University. All rights reserved.
 * 
 * This software was developed to support the EROS secure operating
 * system project (http://www.eros-os.org). The latest version of
 * the OpenCM software can be found at http://www.opencm.org.
 * 
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 * 
 * 3. Neither the name of the The EROS Group, LLC nor the name of
 *    Johns Hopkins University, nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <opencm.h>

Version *
version_create(const char *cfgName, uint64_t rev)
{
  Version *v = (Version *) GC_MALLOC(sizeof(Version));

  ser_init(v, &Version_SerType, Version_SerType.ver);
  v->cfgName = cfgName;
  v->rev = rev;

  SER_MODIFIED(v);

  return v;
}

OC_bool
version_check(const void *v)
{
  return TRUE;
}

void
version_show(const void *vp)
{
  const Version *v = vp;
  
  report(0, "Configuration name:    %s\n", v->cfgName);
  report(0, "Branch version number: %s\n", v->rev);
}

void
version_serialize(SDR_stream *strm, const void *ob)
{
  THROW(ExAssert, "version_serialize() should not have been called");
}

void *
version_deserialize(const DeserializeInfo *di, SDR_stream *strm)
{
  THROW(ExAssert, "version_deserialize() should not have been called");
}

void
version_mark(Repository *r, const void *container,
	     const void *ob, rbtree *memObs)
{
  assert(container == ob);
  /* Do nothing */
}
